@@ -1,86 +1,95 @@
 use std::{collections::HashSet, convert::TryFrom, io::Write};
 
 use anyhow::{Context, Result};
-use structopt::StructOpt;
+use clap::Parser;
 
 use lintrunner::{
     do_init, do_lint, lint_config::get_linters_from_config, path::AbsPath, render::print_error,
-    PathsToLint,
+    PathsToLint, RevisionOpt,
 };
 
-#[derive(Debug, StructOpt)]
+#[derive(Debug, Parser)]
 #[structopt(name = "lintrunner", about = "A lint runner")]
-struct Opt {
+struct Args {
     /// Verbose mode (-v, or -vv to show full list of paths being linted)
-    #[structopt(short, long, parse(from_occurrences))]
+    #[clap(short, long, parse(from_occurrences))]
     verbose: u8,
 
     /// Path to a toml file defining which linters to run
-    #[structopt(long, default_value = ".lintrunner.toml")]
+    #[clap(long, default_value = ".lintrunner.toml")]
     config: String,
 
     /// If set, any suggested patches will be applied
-    #[structopt(short, long)]
+    #[clap(short, long)]
     apply_patches: bool,
 
     /// Shell command that returns new-line separated paths to lint
-    /// (e.g. --paths-cmd 'git ls-files path/to/project')
-    #[structopt(long, conflicts_with = "paths-from")]
+    ///
+    /// Example: --paths-cmd 'git ls-files path/to/project'
+    #[clap(long, conflicts_with = "paths-from")]
     paths_cmd: Option<String>,
 
     /// File with new-line separated paths to lint
-    #[structopt(long)]
+    #[clap(long)]
     paths_from: Option<String>,
 
     /// Lint all files that differ between the working directory and the
     /// specified revision. This argument can be any <tree-ish> that is accepted
     /// by `git diff-tree`
-    #[structopt(long, conflicts_with_all=&["paths", "paths-cmd", "paths-from"])]
+    #[clap(long, short, conflicts_with_all=&["paths", "paths-cmd", "paths-from"])]
     revision: Option<String>,
 
+    /// Lint all files that differ between the merge base of HEAD with the
+    /// specified revision and HEAD. This argument can be any <tree-sh> that is
+    /// accepted by `git diff-tree`
+    ///
+    /// Example: lintrunner -m master
+    #[clap(long, short, conflicts_with_all=&["paths", "paths-cmd", "paths-from", "revision"])]
+    merge_base_with: Option<String>,
+
     /// Comma-separated list of linters to skip (e.g. --skip CLANGFORMAT,NOQA)
-    #[structopt(long)]
+    #[clap(long)]
     skip: Option<String>,
 
     /// Comma-separated list of linters to run (opposite of --skip)
-    #[structopt(long)]
+    #[clap(long)]
     take: Option<String>,
 
     /// If set, lintrunner will render lint messages as JSON, according to the
     /// LintMessage spec.
-    #[structopt(long)]
+    #[clap(long)]
     json: bool,
 
-    #[structopt(subcommand)]
+    #[clap(subcommand)]
     cmd: Option<SubCommand>,
 
     /// Paths to lint.
-    #[structopt(conflicts_with_all = &["paths-cmd", "paths-from"])]
+    #[clap(conflicts_with_all = &["paths-cmd", "paths-from"])]
     paths: Vec<String>,
 
     /// If set, always output with ANSI colors, even if we detect the output is
     /// not a user-attended terminal.
-    #[structopt(long, conflicts_with = "json")]
+    #[clap(long, conflicts_with = "json")]
     force_color: bool,
 }
 
-#[derive(StructOpt, Debug)]
+#[derive(Debug, Parser)]
 enum SubCommand {
     /// Perform first-time setup for linters
     Init {
         /// If set, do not actually execute initialization commands, just print them
-        #[structopt(long, short)]
+        #[clap(long, short)]
         dry_run: bool,
     },
 }
 
 fn do_main() -> Result<i32> {
-    let opt = Opt::from_args();
-    if opt.force_color {
+    let args = Args::parse();
+    if args.force_color {
         console::set_colors_enabled(true);
         console::set_colors_enabled_stderr(true);
     }
-    let log_level = match (opt.verbose, opt.json) {
+    let log_level = match (args.verbose, args.json) {
         // Default
         (0, false) => log::LevelFilter::Info,
         // If just json is asked for, suppress most output except hard errors.
@@ -95,15 +104,15 @@ fn do_main() -> Result<i32> {
     };
     env_logger::Builder::new().filter_level(log_level).init();
 
-    let config_path = AbsPath::try_from(&opt.config)
-        .with_context(|| format!("Could not read lintrunner config at: '{}'", opt.config))?;
-    let skipped_linters = opt.skip.map(|linters| {
+    let config_path = AbsPath::try_from(&args.config)
+        .with_context(|| format!("Could not read lintrunner config at: '{}'", args.config))?;
+    let skipped_linters = args.skip.map(|linters| {
         linters
             .split(',')
             .map(|linter_name| linter_name.to_string())
             .collect::<HashSet<_>>()
     });
-    let taken_linters = opt.take.map(|linters| {
+    let taken_linters = args.take.map(|linters| {
         linters
             .split(',')
             .map(|linter_name| linter_name.to_string())
@@ -112,21 +121,29 @@ fn do_main() -> Result<i32> {
 
     let linters = get_linters_from_config(&config_path, skipped_linters, taken_linters)?;
 
-    let enable_spinners = opt.verbose == 0 && !opt.json;
+    let enable_spinners = args.verbose == 0 && !args.json;
 
-    let paths_to_lint = if let Some(paths_file) = opt.paths_from {
+    let paths_to_lint = if let Some(paths_file) = args.paths_from {
         let path_file = AbsPath::try_from(&paths_file)
             .with_context(|| format!("Failed to find `--paths-from` file '{}'", paths_file))?;
         PathsToLint::PathsFile(path_file)
-    } else if let Some(paths_cmd) = opt.paths_cmd {
+    } else if let Some(paths_cmd) = args.paths_cmd {
         PathsToLint::PathsCmd(paths_cmd)
-    } else if !opt.paths.is_empty() {
-        PathsToLint::Paths(opt.paths)
+    } else if !args.paths.is_empty() {
+        PathsToLint::Paths(args.paths)
     } else {
         PathsToLint::Auto
     };
 
-    match opt.cmd {
+    let revision_opt = if let Some(revision) = args.revision {
+        RevisionOpt::Revision(revision)
+    } else if let Some(merge_base_with) = args.merge_base_with {
+        RevisionOpt::MergeBaseWith(merge_base_with)
+    } else {
+        RevisionOpt::Head
+    };
+
+    match args.cmd {
         Some(SubCommand::Init { dry_run }) => {
             // Just run initialization commands, don't actually lint.
             do_init(linters, dry_run)
@@ -136,10 +153,10 @@ fn do_main() -> Result<i32> {
             do_lint(
                 linters,
                 paths_to_lint,
-                opt.apply_patches,
-                opt.json,
+                args.apply_patches,
+                args.json,
                 enable_spinners,
-                opt.revision,
+                revision_opt,
             )
         }
     }
